package internal

// This file was generated by colf(1); DO NOT EDIT

import (
	"fmt"
	"io"
)

// Colfer configuration attributes
var (
	// ColferSizeMax is the upper limit for serial byte sizes.
	ColferSizeMax = 16 * 1024 * 1024
)

// ColferMax signals an upper limit breach.
type ColferMax string

// Error honors the error interface.
func (m ColferMax) Error() string { return string(m) }

// ColferError signals a data mismatch as as a byte index.
type ColferError int

// Error honors the error interface.
func (i ColferError) Error() string {
	return fmt.Sprintf("colfer: unknown header at byte %d", i)
}

// ColferTail signals data continuation as a byte index.
type ColferTail int

// Error honors the error interface.
func (i ColferTail) Error() string {
	return fmt.Sprintf("colfer: data continuation at byte %d", i)
}

type Header struct {
	SeqID	uint64
	Method	string
	Error	string
	BodySize	uint32
}

// MarshalTo encodes o as Colfer into buf and returns the number of bytes written.
// If the buffer is too small, MarshalTo will panic.
func (o *Header) MarshalTo(buf []byte) int {
	var i int

	if x := o.SeqID; x >= 1<<49 {
		buf[i] = 0 | 0x80
		buf[i+1], buf[i+2], buf[i+3], buf[i+4] = byte(x>>56), byte(x>>48), byte(x>>40), byte(x>>32)
		buf[i+5], buf[i+6], buf[i+7], buf[i+8] = byte(x>>24), byte(x>>16), byte(x>>8), byte(x)
		i += 9
	} else if x != 0 {
		buf[i] = 0
		i++
		for x >= 0x80 {
			buf[i] = byte(x | 0x80)
			x >>= 7
			i++
		}
		buf[i] = byte(x)
		i++
	}

	if l := len(o.Method); l != 0 {
		buf[i] = 1
		i++
		x := uint(l)
		for x >= 0x80 {
			buf[i] = byte(x | 0x80)
			x >>= 7
			i++
		}
		buf[i] = byte(x)
		i++
		i += copy(buf[i:], o.Method)
	}

	if l := len(o.Error); l != 0 {
		buf[i] = 2
		i++
		x := uint(l)
		for x >= 0x80 {
			buf[i] = byte(x | 0x80)
			x >>= 7
			i++
		}
		buf[i] = byte(x)
		i++
		i += copy(buf[i:], o.Error)
	}

	if x := o.BodySize; x >= 1<<21 {
		buf[i] = 3 | 0x80
		buf[i+1], buf[i+2], buf[i+3], buf[i+4] = byte(x>>24), byte(x>>16), byte(x>>8), byte(x)
		i += 5
	} else if x != 0 {
		buf[i] = 3
		i++
		for x >= 0x80 {
			buf[i] = byte(x | 0x80)
			x >>= 7
			i++
		}
		buf[i] = byte(x)
		i++
	}

	buf[i] = 0x7f
	i++
	return i
}

// MarshalLen returns the Colfer serial byte size.
// The error return option is internal.ColferMax.
func (o *Header) MarshalLen() (int, error) {
	l := 1

	if x := o.SeqID; x >= 1<<49 {
		l += 9
	} else if x != 0 {
		l += 2
		for x >= 0x80 {
			x >>= 7
			l++
		}
	}

	if x := len(o.Method); x != 0 {
		l += x
		for x >= 0x80 {
			x >>= 7
			l++
		}
		l += 2
	}

	if x := len(o.Error); x != 0 {
		l += x
		for x >= 0x80 {
			x >>= 7
			l++
		}
		l += 2
	}

	if x := o.BodySize; x >= 1<<21 {
		l += 5
	} else if x != 0 {
		l += 2
		for x >= 0x80 {
			x >>= 7
			l++
		}
	}

	if l > ColferSizeMax {
		return l, ColferMax(fmt.Sprintf("colfer: struct internal.header exceeds %d bytes", ColferSizeMax))
	}
	return l, nil
}

// MarshalBinary encodes o as Colfer conform encoding.BinaryMarshaler.
// The error return option is internal.ColferMax.
func (o *Header) MarshalBinary() (data []byte, err error) {
	l, err := o.MarshalLen()
	if err != nil {
		return nil, err
	}
	data = make([]byte, l)
	o.MarshalTo(data)
	return data, nil
}

// Unmarshal decodes data as Colfer and returns the number of bytes read.
// The error return options are io.EOF, internal.ColferError and internal.ColferMax.
func (o *Header) Unmarshal(data []byte) (int, error) {
	if len(data) == 0 {
		return 0, io.EOF
	}
	header := data[0]
	i := 1

	if header == 0 {
		if i+1 >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		x := uint64(data[i])
		i++

		if x >= 0x80 {
			x &= 0x7f
			for shift := uint(7); ; shift += 7 {
				b := uint64(data[i])
				i++
				if i >= len(data) {
					if i >= ColferSizeMax {
						return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
					}
					return 0, io.EOF
				}

				if b < 0x80 || shift == 56 {
					x |= b << shift
					break
				}
				x |= (b & 0x7f) << shift
			}
		}
		o.SeqID = x

		header = data[i]
		i++
	} else if header == 0|0x80 {
		if i+8 >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		o.SeqID = uint64(data[i])<<56 | uint64(data[i+1])<<48 | uint64(data[i+2])<<40 | uint64(data[i+3])<<32 | uint64(data[i+4])<<24 | uint64(data[i+5])<<16 | uint64(data[i+6])<<8 | uint64(data[i+7])
		header = data[i+8]
		i += 9
	}

	if header == 1 {
		if i >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		x := uint(data[i])
		i++

		if x >= 0x80 {
			x &= 0x7f
			for shift := uint(7); ; shift += 7 {
				if i >= len(data) {
					if i >= ColferSizeMax {
						return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
					}
					return 0, io.EOF
				}
				b := uint(data[i])
				i++

				if b < 0x80 {
					x |= b << shift
					break
				}
				x |= (b & 0x7f) << shift
			}
		}
		if x > uint(ColferSizeMax) {
			return 0, ColferMax(fmt.Sprintf("colfer: internal.header.method size %d exceeds %d bytes", x, ColferSizeMax))
		}
		to := i + int(x)
		if to >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		o.Method = string(data[i:to])

		header = data[to]
		i = to + 1
	}

	if header == 2 {
		if i >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		x := uint(data[i])
		i++

		if x >= 0x80 {
			x &= 0x7f
			for shift := uint(7); ; shift += 7 {
				if i >= len(data) {
					if i >= ColferSizeMax {
						return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
					}
					return 0, io.EOF
				}
				b := uint(data[i])
				i++

				if b < 0x80 {
					x |= b << shift
					break
				}
				x |= (b & 0x7f) << shift
			}
		}
		if x > uint(ColferSizeMax) {
			return 0, ColferMax(fmt.Sprintf("colfer: internal.header.error size %d exceeds %d bytes", x, ColferSizeMax))
		}
		to := i + int(x)
		if to >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		o.Error = string(data[i:to])

		header = data[to]
		i = to + 1
	}

	if header == 3 {
		if i+1 >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		x := uint32(data[i])
		i++

		if x >= 0x80 {
			x &= 0x7f
			for shift := uint(7); ; shift += 7 {
				b := uint32(data[i])
				i++
				if i >= len(data) {
					if i >= ColferSizeMax {
						return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
					}
					return 0, io.EOF
				}

				if b < 0x80 {
					x |= b << shift
					break
				}
				x |= (b & 0x7f) << shift
			}
		}
		o.BodySize = x

		header = data[i]
		i++
	} else if header == 3|0x80 {
		if i+4 >= len(data) {
			if i >= ColferSizeMax {
				return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
			}
			return 0, io.EOF
		}
		o.BodySize = uint32(data[i])<<24 | uint32(data[i+1])<<16 | uint32(data[i+2])<<8 | uint32(data[i+3])
		header = data[i+4]
		i += 5
	}

	if header != 0x7f {
		return 0, ColferError(i - 1)
	}
	if i >= ColferSizeMax {
		return 0, ColferMax(fmt.Sprintf("colfer: internal.header exceeds %d bytes", i, ColferSizeMax))
	}
	return i, nil
}

// UnmarshalBinary decodes data as Colfer conform encoding.BinaryUnmarshaler.
// The error return options are io.EOF, internal.ColferError, internal.ColferTail and internal.ColferMax.
func (o *Header) UnmarshalBinary(data []byte) error {
	i, err := o.Unmarshal(data)
	if i < len(data) && err == nil {
		return ColferTail(i)
	}
	return err
}
